@@ -15,15 +15,27 @@
  *
  * The above copyright notice and this permission notice shall be included in
  * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include <lttng/bug.h>
 #include <unistd.h>
+#include <limits.h>
 
+#ifndef PAGE_SIZE	/* Cygwin limits.h defines its own PAGE_SIZE */
 #define PAGE_SIZE		sysconf(_SC_PAGE_SIZE)
+#endif
+
 #define PAGE_MASK		(~(PAGE_SIZE - 1))
 #define __ALIGN_MASK(v, mask)	(((v) + (mask)) & ~(mask))
-#define ALIGN(v, align)		__ALIGN_MASK(v, (typeof(v)) (align) - 1)
+#define ALIGN(v, align)		__ALIGN_MASK(v, (__typeof__(v)) (align) - 1)
 #define PAGE_ALIGN(addr)	ALIGN(addr, PAGE_SIZE)
 
 /**
